@@ -1,27 +1,27 @@
 package IMDB::Movie;
 
 use strict;
-use vars qw($VERSION $AUTOLOAD);
+use vars qw($VERSION $AUTOLOAD @MATCH);
 
 use Carp;
 use LWP::Simple;
 use HTML::TokeParser;
 use Data::Dumper;
 
-$VERSION = '0.09';
-use constant TITLE => 'http://www.imdb.com/title/tt';
-use constant FIND  => 'http://www.imdb.com/Find?select=All&for=';
+$VERSION = '0.10';
 
 sub new {
-	my ($class,$id) = @_;
+	my ($class,$id,$site) = @_;
+	chomp($id);
 	carp "can't instantiate $class without id or keyword" unless $id;
-	$id = sprintf("%07d",$id) unless length($id) == 8 or $id =~ /\D/;
+	$id = sprintf("%07d",$id) unless length($id) == 7 or $id =~ /\D/;
+	$site ||= 'www';
 
-	my $parser = _get_toker($id);
+	my $parser = _get_toker($id,$site);
 	my ($title,$year);
 
 	# get the ball rolling here
-	($parser,$title,$year) = _title_year($parser,$id);
+	($parser,$title,$year) = _title_year($parser,$id,$site);
 
 	$title =~ tr/"//d;
 
@@ -37,6 +37,8 @@ sub new {
 		writers     => _person($parser),
 		genres      => _genre($parser),
 		user_rating => _user_rating($parser),
+		cast        => _cast($parser),
+		matches     => \@MATCH,
 	};
 	return bless $self, $class;
 }
@@ -57,8 +59,10 @@ sub as_HTML_Template {
 	my $clone = Clone::clone($self);
 	my %d = %{$clone->directors};
 	my %w = %{$clone->writers};
+	my %c = %{$clone->cast};
 	$clone->{directors} = [ sort{$a->{id}<=>$b->{id}} values %d ];
 	$clone->{writers}   = [ sort{$a->{id}<=>$b->{id}} values %w ];
+	$clone->{cast}      = [ sort{$a->{id}<=>$b->{id}} values %c ];
 	$clone->{genres}    = [ map {name => $_}, @{$clone->genres} ];
 	return %$clone;
 }
@@ -79,18 +83,17 @@ sub DESTROY {}
 sub _merge_names { [sort map "$_->{last_name}, $_->{first_name}", values %{shift->{+shift}} ] }
 
 sub _title_year {
-	my $parser = shift;
-	my $id     = shift;
+	my ($parser,$id,$site) = @_;
 	my ($title,$year);
 
 	$parser->get_tag('title');
 	$title = $parser->get_text();
 
 	if ($title eq 'IMDb name and title search') {
-		$id = _get_lucky($parser,$id);
+		$id = _get_lucky($parser);
 
 		# start over
-		$parser = _get_toker($id);
+		$parser = _get_toker($id,$site);
 		$parser->get_tag('title');
 		$title = $parser->get_text();
 	}
@@ -102,16 +105,18 @@ sub _title_year {
 }
 
 sub _get_lucky {
-	my ($parser,$thing) = @_;
+	my ($parser) = @_;
 	my ($tag,$id);
 
 	while ($tag = $parser->get_tag('a')) {
 		my $href = $tag->[1]->{href};
 		next unless $href;
-		last if ($id) = $href =~ /\/title\/tt(\d{7})/;
+		if (($id) = $href =~ /\/title\/tt(\d{7})/) {
+			push @MATCH, {id => $id, title => $parser->get_text};
+		}
 	}
 
-	return $id;
+	return $MATCH[0]{id};
 }
 
 sub _id {
@@ -148,7 +153,6 @@ sub _image {
 
 sub _person {
 	my $parser = shift;
-	#my ($tag,@name);
 	my ($tag,%name);
 
 	# skip
@@ -170,7 +174,6 @@ sub _person {
 		redo;
 	}
 
-	#return [ unique(@name) ];
 	return {%name};
 }
 
@@ -209,11 +212,43 @@ sub _user_rating {
 	return $rating;
 }
 
+sub _cast {
+	my $parser = shift;
+	my ($tag,%name);
+
+	# skip
+	while ($tag = $parser->get_tag('b')) {
+		if ($tag->[1]{class}) {
+			last if $tag->[1]{class} eq 'blackcatheader';
+		}
+	}
+
+	while ($tag = $parser->get_tag('a')) {
+		my $href = $tag->[1]{href};
+		if ($href) {
+			if ($href eq 'fullcredits') {
+				last;
+			} else {
+				my ($id) = $href =~ /\/name\/nm(\d+)/;
+				if ($id) {
+					my $name = $parser->get_text;
+					$name = reverse $name;
+					my ($l,$f) = map { scalar reverse $_} split(' ',$name,2);
+					$name{$id} = { id => $id, last_name => $l, first_name => $f };
+				}
+			}
+		}
+	}
+	return {%name};
+}
+
 sub _get_toker {
-	my $id = shift;
+	my ($id,$site) = @_;
+	my $title_url = "http://$site.imdb.com/title/tt";
+	my $find_url  = "http://$site.imdb.com/Find?select=All&for=";
 	my $url = ($id =~ /\D/)
-		? FIND  .$id
-		: TITLE . $id
+		? $find_url  .$id
+		: $title_url . $id
 	;
 
 	my $content = get($url) or carp "can't connect to server";
@@ -225,7 +260,6 @@ sub unique {
 	grep(!$seen{$_}++, @_);
 }
 
-
 1;
 
 =pod
@@ -278,6 +312,12 @@ the IMDB identification number, but you can pass the name of the
 movie with moderate success. Note that this causes an extra page
 fetch as IMDB::Movie parses the search results.
 
+You can also specify which 'site' you want to search:
+
+  my $movie = IMDB::Movie->new('Alien','uk');
+
+This will search B<uk.imdb.com> instead of B<www.imdb.com>, the default.
+
 =item B<title>
 
   my $title = $movie->title;
@@ -332,6 +372,22 @@ Return an anonymous hash reference whose keys are IMDB
 name id's and whose values are anonymous hash references
 containing first and last name key/value pairs.
 
+=item B<cast>
+
+  my %cast = %{$movie->cast};
+  for my $id (keys %cast) {
+     print $cast{$id}{first};
+     print $cast{$id}{last};
+  }
+
+Return an anonymous hash reference whose keys are IMDB
+name id's and whose values are anonymous hash references
+containing first and last name key/value pairs.
+
+This is for the First Billed cast only, that is, the module
+parses out the name from the first page. Possibly in the
+future i will parse out the entire crew, but it's doubtful.
+
 =item B<genres>
 
   my @genres = @{$movie->genres};
@@ -350,6 +406,35 @@ Returns the current IMDB user rating as is.
 
 Returns the url of the image used for this Movie at imdb.com
 
+=item B<matches>
+
+  my @match = @{$movie->matches};
+
+Returns a list of hashes (LoH) containing 'id' and 'title'
+key/value pairs for all title matches returned when a seach
+by title was performed. For example:
+
+  use IMDB::Movie;
+  use CGI qw(:standard);
+  my $movie = IMDB::Movie->new('Terminator');
+  my @match = @{$movie->matches};
+
+  if (@match) {
+     print (
+        p('The following matches were found:'),
+        ol(
+           li([ map
+              a(
+                 {href => "http://imdb.com/title/tt$_->{id}"},
+                 $_->{title}
+              ), @match
+           ])
+        ),
+     );
+  }
+
+will produce an HTML ordered list of anchored links.
+
 =item B<as_HTML_Template>
 
   my %t_movie = $movie->as_HTML_Template;
@@ -377,7 +462,7 @@ irresponsible manor.
 Also, screen-scraping a web site does not make for a long living
 application. Any changes to IMDB's design could potentially break
 this module. I give no garuantee that i will maintain this module,
-but i will garuantees that i may just delete this module with no
+but i will garuantee that i may just delete this module with no
 notice. 
 
 =head1 COPYRIGHT
